#!/bin/bash

#!/bin/bash
clear

echo "Hello.  I am going to walk you through the process of adding a new device"
echo "type to the carrot system.  Note that I will only provide a skeleton for"
echo "the device - you will be responsible for adding the remaining code and"
echo "and ensuring the correctness of the auto-generated code."

echo ""
echo ""
echo ""

echo "I am going to being by asking you a few questions about yourself and the"
echo "device module so that the proper name can be provided"
echo ""
echo -n "Please enter the device main class name (must start with uppercase): "
read DEVICE_NAME
echo -n "Please enter the device netlist name (must be all lowercase): "
read NETLIST_NAME
echo -n "Please enter a short description of the device: "
read DEVICE_DESCRIPTION
echo "I need your name to be used author of the device"
echo -n "Please enter your name: "
read DEVICE_AUTHOR
echo -n "Please enter a URL for this project or yourself: "
read DEVICE_ADDRESS



echo "****************************************************************"
echo "Please verify the following information as correct:"
echo "Device Class Name:    ${DEVICE_NAME}"
echo "Device Netlist Name:  ${NETLIST_NAME}"
echo "Device Description:   ${DEVICE_DESCRIPTION}"
echo "Device Author:        ${DEVICE_AUTHOR}"
echo "Device Address:       ${DEVICE_ADDRESS}"
echo "****************************************************************"
echo -n "Continue with device creation [N/y]: "
read SCRIPT_CONTINUE

if [ "$SCRIPT_CONTINUE" != "y" -a "$SCRIPT_CONTINUE" != "Y" ]
then
	echo "Aborted"
	exit
fi

mkdir devs/$DEVICE_NAME

echo -n "Creating Makefile.am"

rm -f devs/$DEVICE_NAME/Makefile.am
echo "###########################################################################" >> devs/$DEVICE_NAME/Makefile.am
echo "## Makefile.am -- Process this file with automake to produce Makefile.in" >> devs/$DEVICE_NAME/Makefile.am
echo "##" >> devs/$DEVICE_NAME/Makefile.am
echo "## Copyright (C) `date +%Y`, $DEVICE_AUTHOR" >> devs/$DEVICE_NAME/Makefile.am
echo "###########################################################################" >> devs/$DEVICE_NAME/Makefile.am
echo "" >> devs/$DEVICE_NAME/Makefile.am
echo "MAINTAINERCLEANFILES = Makefile.in *~" >> devs/$DEVICE_NAME/Makefile.am
echo "" >> devs/$DEVICE_NAME/Makefile.am
echo "INCLUDES=-I../../include -I../" >> devs/$DEVICE_NAME/Makefile.am
echo "LIBADD = -lckt -L../../ckt" >> devs/$DEVICE_NAME/Makefile.am
echo "" >> devs/$DEVICE_NAME/Makefile.am
echo 'plugindir=$(pkglibdir)/devs' >> devs/$DEVICE_NAME/Makefile.am
echo "" >> devs/$DEVICE_NAME/Makefile.am
echo "plugin_LTLIBRARIES = lib$DEVICE_NAME.la" >> devs/$DEVICE_NAME/Makefile.am
echo "" >> devs/$DEVICE_NAME/Makefile.am
echo "lib${DEVICE_NAME}_la_SOURCES =\\" >> devs/$DEVICE_NAME/Makefile.am
echo "	$DEVICE_NAME.h $DEVICE_NAME.cc" >> devs/$DEVICE_NAME/Makefile.am
#echo "	../../include/carrot/MacroElement.h \\" >> devs/$DEVICE_NAME/Makefile.am
#echo "	../../include/carrot/ParmSet.h" >> devs/$DEVICE_NAME/Makefile.am

echo ".....DONE!"


echo -n "Creating $DEVICE_NAME.h"

rm -f devs/$DEVICE_NAME/$DEVICE_NAME.h
echo "//Copyright `date +%Y`, $DEVICE_AUTHOR"   >> devs/$DEVICE_NAME/$DEVICE_NAME.h
echo "#ifndef ${DEVICE_NAME}_H" >> devs/$DEVICE_NAME/$DEVICE_NAME.h
echo "#define ${DEVICE_NAME}_H 1"  >> devs/$DEVICE_NAME/$DEVICE_NAME.h
echo ""   >> devs/$DEVICE_NAME/$DEVICE_NAME.h
echo "#include <carrot/MacroElement.h>"  >> devs/$DEVICE_NAME/$DEVICE_NAME.h
echo ""   >> devs/$DEVICE_NAME/$DEVICE_NAME.h
echo "class $DEVICE_NAME : public MacroElement"   >> devs/$DEVICE_NAME/$DEVICE_NAME.h
echo "{"   >> devs/$DEVICE_NAME/$DEVICE_NAME.h
echo "public:"   >> devs/$DEVICE_NAME/$DEVICE_NAME.h
echo ""   >> devs/$DEVICE_NAME/$DEVICE_NAME.h
echo "  $DEVICE_NAME(ParmSet *pset, const std::string& name);"   >> devs/$DEVICE_NAME/$DEVICE_NAME.h
echo ""   >> devs/$DEVICE_NAME/$DEVICE_NAME.h
echo "  virtual ~$DEVICE_NAME();" >> devs/$DEVICE_NAME/$DEVICE_NAME.h
echo ""  >> devs/$DEVICE_NAME/$DEVICE_NAME.h
echo "  virtual void init();" >> devs/$DEVICE_NAME/$DEVICE_NAME.h
echo ""  >> devs/$DEVICE_NAME/$DEVICE_NAME.h
echo "private:"  >> devs/$DEVICE_NAME/$DEVICE_NAME.h
echo ""  >> devs/$DEVICE_NAME/$DEVICE_NAME.h
echo "};"  >> devs/$DEVICE_NAME/$DEVICE_NAME.h
echo ""  >> devs/$DEVICE_NAME/$DEVICE_NAME.h
echo ""  >> devs/$DEVICE_NAME/$DEVICE_NAME.h
echo "#endif"   >> devs/$DEVICE_NAME/$DEVICE_NAME.h

echo ".....DONE!"


echo -n "Creating $DEVICE_NAME.cc"

#data structures

rm -f devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "#include \"$DEVICE_NAME.h\"" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
#echo "#include <carrot/Circuit.h>" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "static ItemInfo ${DEVICE_NAME}_info = {" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  \"$NETLIST_NAME\"," >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  \"$DEVICE_DESCRIPTION\"," >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  \"$DEVICE_AUTHOR\"" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  \"$DEVICE_ADDRESS\"" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "};" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "// Double Parameters" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "// Make sure enum and ParInfo definitions are in the same order!" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "enum {P1, P2, MAX_DPARM_INDEX};" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "//Insert ParmInfo values here" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "static ParamHandler<double>::ParmInfo ${DEVICE_NAME}_dparams[] = { ">> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "{\"p1\", \"Parameter 1 (units)\", false, 1e-3}," >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "{\"p2\", \"Parameter 2 (units)\", false, 0.}" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "};" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "// Uncomment for integer parameters" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "//enum {MAX_IPARM_INDEX};" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "//static ParamHandler<int>::ParmInfo ${DEVICE_NAME}_iparams[] = {  };" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc

#library loading mechanism

echo "//Library loading mechanism" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "extern \"C\" {" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  const char * ModuleName(){" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "    return ${DEVICE_NAME}_info.type.c_str();" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  }" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
#echo "" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  ParmSet * newParmSet(const char * model){" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "    return new ParmSet(model," >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "		         &${DEVICE_NAME}_info," >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "		         ${DEVICE_NAME}_dparams, MAX_DPARM_INDEX); " >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "//		         ${DEVICE_NAME}_iparams, MAX_IPARM_INDEX);">> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  }" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
#echo "" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  void delParmSet(ParmSet * p){" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "    delete p;" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "    p=0;" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  }" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
#echo "" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  MacroElement * newObject(ParmSet * p, const char * name){" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "    return new $DEVICE_NAME(p, name);" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  }" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  void delObject(MacroElement * e){" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "    delete e;" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "    e = 0;" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  }" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "};" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc

#Actual device class/methods

echo "$DEVICE_NAME::$DEVICE_NAME(ParmSet *pset, const std::string& name)" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  :MacroElement(pset, name, 2)" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "{" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  //Constructor code goes here (above, 2 is the number of terminals)" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "}" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "$DEVICE_NAME::~$DEVICE_NAME()" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "{" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  //Destructor code goes here" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "}" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "void $DEVICE_NAME::init()" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "{" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  // access to double parameters" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  double *dparm = getParmSet()->getDHandler()->getParmVec();" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  // Check required parameters" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  getParmSet()->checkParms();" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  // Check external terminal connections" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  assert(checkTerms());" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  // Set number of internal nodes" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  setNInternalNodes(0);" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  // 1. Create individual CSs" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  // LinearVCCS* lvccs = new LinearVCCS(dparm[P1]);" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  // Insert into MacroElement internal lists" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  // insert(lvccs);" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  // 2. connect internal terminals." >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  // Vector with internal terminal connections (must be static)" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  // In the example output and controlling terminals are" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  // connected to external terminals 0 and 1." >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  // static const int tvec[4] = {0,1,0,1};" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "  // lvccs->setTermNum(tvec);" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "}" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc
echo "" >> devs/$DEVICE_NAME/$DEVICE_NAME.cc

echo ".....DONE!"


echo -n "Modifying devs Makefile.am"

cp devs/Makefile.am devs/Makefile.am.bak
cat devs/Makefile.am.bak | sed -e "/SUBDIRS/a$DEVICE_NAME \\\\" > devs/Makefile.am

echo ".....DONE!"



echo -n "Modifying root configure.in"

cp configure.in configure.in.bak
cat configure.in.bak | sed -e "/AC_OUTPUT/adevs/$DEVICE_NAME/Makefile" > configure.in

echo ".....DONE!"


echo "Now updating all configure scripts and makefiles"
aclocal-1.9
automake-1.9
autoconf
