/*  Copyright Garrett Kajmowicz, 2004.  Licenced under the GNU GPL.
 *  This file defines a set of functions which will load the modules for 
 *  object definitions and code.
 *
 *
 */


#ifndef AnalysisLoader_CPP
#define AnalysisLoader_CPP

#include <AnalysisLoader.h>

//The following header file is needed to represent the base class
// of all of the elements in use
#include <carrot/Analysis.h>

//Needed for Unix library routines - cross platform how?
// Conforms to POSIX 1003.1-2003
#include <dlfcn.h>

#include <sys/types.h>
#include <dirent.h>
#include <cstring>
#include <cstdio>

AnalysisMap openAnalysis(const std::string path, bool printflag,
			 AnalysisMap retval)
{
  AnalysisFunctions temp;

  //Open up the required directory to search for modules
  if (printflag) {
    printf("****************************************************************************\n");
    printf("Loading analysis libraries from: %s\n\n", path.c_str() );
  }
  DIR * d = opendir(path.c_str());
  if(d == NULL){
    return retval;
  }

  struct dirent * f;

  //Get the name of each file
  do{
    f = readdir(d);
    if(f != 0 ){
      int stringlength = strlen(f->d_name);

      //Check to make sure that the file extansion is ".so"
      if( strncmp(f->d_name + stringlength - 3, ".so", 3) == 0 ){
        std::string filename = path + "/";
        filename += f->d_name;

        //Create a dl handle for the library.  Use RTLD_NOW to prevent problems
        dlerror();    //Flush errors
        temp.handle = dlopen(filename.c_str(), RTLD_NOW);

        if(temp.handle != 0){
          //Now we just get the function pointers that we need and we are flying
          temp.AnalysisName = (const char *(*)())dlsym(temp.handle, "AnalysisName");
          temp.newParmSet = (ParmSet *(*)(const char *))dlsym(temp.handle, "newParmSet");
          temp.delParmSet = (void (*)(ParmSet *))dlsym(temp.handle, "delParmSet");
          temp.newObject = (Analysis*(*)(ParmSet*))dlsym(temp.handle, "newObject");
          temp.delObject = (void (*)(Analysis*))dlsym(temp.handle, "delObject");

          //Assume that all pointers will not be 0 - not technicaly correct but usually safe

          if(temp.AnalysisName !=0 && temp.newParmSet != 0 && temp.delParmSet != 0 && temp.newObject !=0 && temp.delObject != 0){
            //Add to matrix
            retval[temp.AnalysisName()] = temp;
	    if (printflag) 
	      printf("Added: %s\n", filename.c_str() );
	  }else{
            fprintf(stderr, 
		    "Failed to add: %s - not all functions were specified\n", 
		    filename.c_str() );
          }

        }else{
          fprintf(stderr, "Failed to DLOPEN %s\n", filename.c_str() );
          fprintf(stderr, "Reason: %s\n", dlerror());
        }

      }
    }
    
  }while(f != 0);

  //Make sure that we open up the handle with immediate name resolution
  // There is no sense in waiting

  //temp.handle = dlopen("filename", RTLD_NOW);

  if (printflag) 
    printf("****************************************************************************\n\n");
  
  return retval;
}


void closeAnalysis(AnalysisMap & m){
  return;
}


#endif
