// This may look like C code, but it is really -*- C++ -*-
//
// Envelope and regular HB analysis
//
// 

#ifndef EnvelopeHB_H
#define EnvelopeHB_H

#include <carrot/Analysis.h>
#include <carrot/UMFPackDI.h>
#include <string>
#include <complex>
#include <fftw3.h>

typedef std::complex<double> double_complex;

class Circuit;

class EnvelopeHB : public Analysis
{
public:

  // ParmSet takes care of parameter handling
  EnvelopeHB(ParmSet* pset1) : Analysis(pset1) {}

  virtual ~EnvelopeHB() {}

  // The main method to actually run the analysis
  // (Decide what to do with the results)
  virtual void run(Circuit* cir, OutputManager* om);

  // To be used by Newton solver
  int fillNAMandRHS(UMFPackDI& nam, double* svec, double* uvec_j, 
		    const double& beta, const double& gmin);

private:
  
  // Just normalise while copying
  void normAndCopy(double_complex* source, 
		   double_complex* dest1,
		   const int& nsamples, const int& nh);

  // Denormalise source and copy to dest (for inverse transform)
  void denormAndCopy(double_complex* source, 
		     double_complex* dest,
		     const int& nh);

  void addQuad(UMFPackDI& nam, 
	       int row1, int row2,
	       int col1, int col2,
	       const double& g);

  void addQuad(UMFPackDI& nam, 
	       int row1, int row2,
	       int col1, int col2,
	       const double_complex& g);

  void c2d(double_complex* uvec, double* uvec_d, 
	   const int& nrcs, const int& num_HB);

  void d2c(double_complex* uvec, double* uvec_d, 
	   const int& nrcs, const int& num_HB);

  // Variables use during analysis
  int nrcs;
  LinVCCSVector linvccs;
  IndepCSVector indcs;
  LinVCQSVector linvcqs;
  GenVCCSVector genvccs;
  GenVCQSVector genvcqs;
  double_complex *svec_ground1;
  int padsize;
  int num_HB;
  double* Omega;
  double_complex *uvec;
  double_complex *svec;
  //linear charge source (linear capacitor)
  double cFactor;
  double *put1;
  double_complex *put2;
  fftw_plan p1;
  fftw_plan p2;
  int N;
  double *uvec_td;
  double *vj1;
  double *in_dj;
  double *dj;
  int nCharges;
  double_complex *q;
  double *B;

};

#endif
