#include <carrot/ElementNode.h>
#include <carrot/Terminal.h>

// 6 seems to be a reasonable start value
ElementNode::ElementNode(const unsigned& nterms) 
  : termVec(nterms)
{
  // initialise terminal vector to zero
  for (std::vector<Terminal*>::iterator cterm = termVec.begin();
       cterm != termVec.end(); cterm++)
    (*cterm) = NULL;
}

ElementNode::~ElementNode()
{
  // Circuit takes care of inconsistencies (tells terminals if
  // necessary)
}

void ElementNode::setNTerms(const unsigned& nterms)
{
  unsigned oldsize = termVec.size();
  termVec.resize(nterms);
  if (oldsize < nterms) {
    // initialise new elements to zero
    for (std::vector<Terminal*>::iterator cterm = termVec.begin() + oldsize;
	 cterm != termVec.end(); cterm++)
      (*cterm) = NULL;
  }
}

void ElementNode::link(const unsigned& index, Terminal* newterm)
{
  assert(index < termVec.size());
  Terminal *&term_p = termVec[index];
  // First, tell the current terminal that we are no longer attached.
  if (term_p)
    term_p->unlink(this);
  term_p = newterm;
  // Link new terminal to us
  newterm->link(this);
}

void ElementNode::unlink(Terminal* oldterm)
{
  for (std::vector<Terminal*>::iterator cterm = termVec.begin();
       cterm != termVec.end(); cterm++)
    if ((*cterm) == oldterm)
      (*cterm) = NULL;
}

bool ElementNode::checkTerms()
{
  for (std::vector<Terminal*>::iterator cterm = termVec.begin();
       cterm != termVec.end(); cterm++)
    if ((*cterm) == NULL)
      return false;
  return true;
}
