#include <carrot/MacroElement.h>

template <typename VECTYPE>
void MacroElement::clearItems(VECTYPE &vec)
{
  unsigned iend = vec.size();
  for (unsigned i(0); i < iend; i++) {
    //cir->remove(vec[i]);
    delete vec[i];
  }
}


MacroElement::~MacroElement() 
{
  // remove us from the ParmSet vector
  pset->removeME(this);
  // Now clean up internal subcircuit
  // First: remove CS from circuit
  clearItems(lvcCSvec);
  clearItems(flvcCSvec);
  clearItems(lvcQSvec);
  clearItems(genVCCSvec);
  clearItems(tdCSvec);
  clearItems(genVCQSvec);
  clearItems(indepCSvec);
}


void MacroElement::initAllCS()
{
  // Go through all gen. VCCS in MacroElement, call init() 
  for (GenVCCSVector::iterator i = genVCCSvec.begin(); 
       i < genVCCSvec.end(); i++)
    (*i)->init();
  // Go through all time-dalayed VCCS in MacroElement, call init() 
  for (TDCSVector::iterator i = tdCSvec.begin(); 
       i < tdCSvec.end(); i++)
    (*i)->init();
  // Go through all gen. VCQS in MacroElement, call init() 
  for (GenVCQSVector::iterator i = genVCQSvec.begin(); 
       i < genVCQSvec.end(); i++)
    (*i)->init();
  // Go through all indep. CS in MacroElement, call init() 
  for (IndepCSVector::iterator i = indepCSvec.begin(); 
       i < indepCSvec.end(); i++)
    (*i)->init();
  // Go through all Linear CS in MacroElement, call init() 
  for (LinVCCSVector::iterator i = lvcCSvec.begin(); 
       i < lvcCSvec.end(); i++)
    (*i)->init();
  // Go through all Linear QS in MacroElement, call init() 
  for (LinVCQSVector::iterator i = lvcQSvec.begin(); 
       i < lvcQSvec.end(); i++)
    (*i)->init();
}
