#include <carrot/ParmSet.h>
// #include <cstring>
#include <string>
#include <iostream>

// **********************************************************************
// ParamHandler members first
// **********************************************************************

template <typename T>
ParamHandler<T>::ParamHandler(ParmInfo* parm_desc, const unsigned& nparms)
  : nparms(nparms), parm_desc(parm_desc), parm_set(nparms), parm(0)
{
  if (nparms) {
    parm = new T[nparms];
//    parm_set = new bool[nparms];
    // Initialise parameters, etc.
    for (unsigned i(0); i < nparms; i++) {
      parm[i] = parm_desc[i].defaultValue;
      parm_set[i] = false;
    }
  }
//  else {
//    parm = NULL;
//    parm_set = NULL;
//  }
}

template <typename T>
ParamHandler<T>::ParamHandler(const ParamHandler<T> & p)
  : nparms(p.nparms), parm_desc(p.parm_desc), parm_set(p.parm_set), parm(0)
{
  if (nparms) {
    parm = new T[nparms];
//    parm_set = new bool[nparms];
    // Initialise parameters, etc.
    for (unsigned i(0); i < nparms; ++i) {
      setParm(i, p.parm[i]);
//      parm_set[i] = p.parm_set[i];
    }
  }
}

template <typename T>
ParamHandler<T>::~ParamHandler()
{
  if (nparms) {
//    delete [] parm_set;
    delete [] parm;
    parm = 0;
  }
}

template <typename T>
ParamHandler<T> & ParamHandler<T>::operator=(const ParamHandler<T> & rs)
{
  if(&rs == this){
    return *this;
  }

  if(nparms != rs.nparms){
    delete [] parm;
    if(rs.nparms == 0 ){
      parm = 0;
    }else{
      parm = new T[rs.nparms];
    }
  }

  nparms = rs.nparms;
  parm_desc = rs.parm_desc;
  parm_set = rs.parm_set;
//  delete [] parm_set;
//  parm_set = 0;
//  delete [] parm;
//  parm = 0;

//  if (nparms) {
//    parm = new T[nparms];
//    parm_set = new bool[nparms];
//    parm = rs.parm;
//    parm_set = rs.parm_set;
    // Initialise parameters, etc.
    for (unsigned i(0); i < nparms; ++i) {
      setParm(i, rs.parm[i]);
//      parm_set[i] = rs.parm_set[i];
    }
//  }
  return *this;
}

  
template <typename T>
unsigned ParamHandler<T>::getIndex(const std::string& parName) const
{
  unsigned i(0);
  for (; i < nparms; i++){
//    if (!strcmp(parName, parm_desc[i].name)) {
    if (parName == parm_desc[i].name) {
      break;
    }
  }
  return i;
}

template <typename T>
void ParamHandler<T>::setParm(const unsigned& index, const T &parvalue)
{
  assert(index < nparms);
  parm[index] = parvalue;
  parm_set[index] = true;
}

template <typename T>
void ParamHandler<T>::checkParms() const
{
  // Just check that required parameters are indeed set.
  // Here we should throw an exceptio if something goes wrong.
  for (unsigned i(0); i < nparms; i++)
    if (parm_desc[i].required && !parm_set[i])
      assert(false);
}


// **********************************************************************
// ParmSet members start
// **********************************************************************

// First, instantiate necessary templates
template class ParamHandler<double>;
template class ParamHandler<int>;
template class ParamHandler<std::string>;


ParmSet::ParmSet(const std::string& name,
		 ItemInfo* i_info, 
		 ParamHandler<double>::ParmInfo *dparam_desc, 
		 const unsigned numdparms,
		 ParamHandler<int>::ParmInfo *iparam_desc, 
		 const unsigned numiparms,
		 ParamHandler<std::string>::ParmInfo *sparam_desc, 
		 const unsigned numsparms)
  : NamedItem(name),
    i_info(i_info), 
    dhandler(dparam_desc, numdparms),
    ihandler(iparam_desc, numiparms),
    shandler(sparam_desc, numsparms)
{
  // Nothing else to do
}


bool ParmSet::askAbout(ParmType& type, unsigned& index, 
		       const std::string& parName) const
{
  // We do not need to check for handlers with zero parameters (return
  // index is zero)

  // Check for doubles first
  index = dhandler.getIndex(parName);
  if (index < dhandler.getNParms()) {
    type = DOUBLE;
    return true;
  }
  // Now Integers
  index = ihandler.getIndex(parName);
  if (index < ihandler.getNParms()) {
    type = INT;
    return true;
  }
  // Now strings
  index = shandler.getIndex(parName);
  if (index < shandler.getNParms()) {
    type = STRING;
    return true;
  }
  // If control reaches here then the parameter does not exist
  // This should throw an exception
  // assert(false);
  return false;
}

void ParmSet::setParm(const unsigned& index, const double& parvalue)
{
  dhandler.setParm(index, parvalue);
}
void ParmSet::setParm(const unsigned& index, const int& parvalue)
{
  ihandler.setParm(index, parvalue);
}
void ParmSet::setParm(const unsigned& index, const std::string& parvalue)
{
  //FIXME by using a real function parameter
  shandler.setParm(index, parvalue);
}

void ParmSet::print()
{
  std::cout << std::endl << "--- Model: " << getName() 
	    << " --- Type: " << getInfo()->type << std::endl;
  ParamHandler<double> *dph = getDHandler();
  if (dph) {
    int nparms = dph->getNParms();
    double *pvec = dph->getParmVec();
    for (int j(0); j<nparms; j++){
      std::cout << dph->getParmDesc(j).name << " = " << pvec[j] << std::endl;
    }
  }
  ParamHandler<int> *iph = getIHandler();
  if (iph) {
    int nparms = iph->getNParms();
    int *pvec = iph->getParmVec();
    for (int j(0); j<nparms; j++){
      std::cout << iph->getParmDesc(j).name << " = " << pvec[j] << std::endl;
    }
  }
  ParamHandler<std::string> *cph = getSHandler();
  if (cph) {
    int nparms = cph->getNParms();
    std::string *pvec = cph->getParmVec();
    for (int j(0); j<nparms; j++){
      std::cout << cph->getParmDesc(j).name << " = " << pvec[j] << std::endl;
    }
  }
}
  
// Check element parameters. 
void ParmSet::checkParms() const
{
  // Remember to add exception throwing to this
  dhandler.checkParms();
  ihandler.checkParms();
  shandler.checkParms();
}

// Do data comparison
bool ParmSet::operator==(ParmSet & rs)
{
//  if( strcmp(rs.getInfo()->type, getInfo()->type) !=0 ){
  if( rs.getInfo()->type != getInfo()->type ){
    //Not for same type of element
    return false;
  }

//  if( strcmp(rs.getInfo()->description, getInfo()->description) !=0 ){
  if( rs.getInfo()->description != getInfo()->description ){
    //Not for same type of element
    return false;
  }

  if(dhandler != rs.dhandler){
    return false;
  }

  if(ihandler != rs.ihandler){
    return false;
  }

  if(shandler != rs.shandler){
    return false;
  }

  return true;
}

bool ParmSet::operator!=(ParmSet & rs)
{
  //Simply invert operator== for the result we want
  if(operator==(rs)){
    return false;
  }
  return true;
}


ParmSet & ParmSet::operator=(const ParmSet & rs)
{
  if(&rs == this){
    return *this;
  }

  dhandler = rs.dhandler;
  ihandler = rs.ihandler;
  shandler = rs.shandler;
  i_info = rs.i_info;
  meVec = rs.meVec;  
  return *this;
}
