
#include <carrot/OutputManager.h>
#include <carrot/Options.h>
#include <cstdio>

OutputManager::OutputManager()
  : ready(false), tv(), varunit("-"), analysistype("none"), sv()
{
  // Do nothing
}

OutputManager::~OutputManager()
{
  // Clean up the handlers
  std::vector<SampleHolder*>::iterator svi = sv.begin();
  std::vector<SampleHolder*>::iterator svend = sv.end();
  while (svi != svend) {
    delete (*svi);
    svi++;
  }
}

class TerminalRCCompare
{
public:
  inline bool operator()(Terminal* x, Terminal* y)
  {
    return (x->getRC()) < (y->getRC());
  }
};

#include <algorithm>

void OutputManager::getRowNumbers(Circuit* cir, std::vector<int>& iv)
{
  assert(!ready);
  // If no variables were selected, select all by default (except ground)
  if (!tv.size()) {
    // Put all Terminals there
    TerminalVector tv1(cir->getTermVec());
    // Erase ground terminal and internal terminals if requested
    TerminalVector::iterator tvi = tv1.begin() + 1; // Remove ground
    TerminalVector::iterator tvend = tv1.end();
    while (tvi != tvend) {
      if(options.getIHandler()->getParmVec()[OPT_INT_TERMS] 
	 || !(*tvi)->isInternal())
	tv.push_back(*tvi);
      tvi++;
    }
  }

  // Sort by RC (to improve efficiency when writing results)
  TerminalRCCompare tc;
  std::sort(tv.begin(), tv.end(), tc);
  // Make sure iv has the right number of elements
  iv.resize(tv.size());
  // Fill iv
  TerminalVector::iterator tvi = tv.begin();
  TerminalVector::iterator tvend = tv.end();
  std::vector<int>::iterator ivi = iv.begin();
  std::vector<int>::iterator ivend = iv.end();
  while (tvi != tvend) {
    *ivi = (*tvi)->getRC();
    // Make sure the terminal info is available
    assert((*ivi) > -1);
    ivi++;
    tvi++;
  }
  ready = true;
}

void OutputManager::write()
{
  assert(ready);

  printf("\nWriting output (%s) ... ", analysistype.c_str());
  // Process output for each terminal at a time
  int nterms = tv.size();
  for (int i(0); i<nterms; i++) {
    // Open output file for this terminal
    std::string filename = tv[i]->getName() + "." + analysistype;
    FILE* output_F = fopen(filename.c_str(), "w");
    if(!output_F) {
      fprintf(stderr, "Error: couldn't open output file.\n");
      exit(1);
    }
    // Output info for all samples
    std::vector<SampleHolder*>::iterator svi = sv.begin();
    std::vector<SampleHolder*>::iterator svend = sv.end();
    while (svi != svend) {
      std::string value;
      (*svi)->writeNode(i, value);
      fprintf(output_F, "%s\n", value.c_str());
      svi++;
    }
    fclose(output_F);
  }
  printf("done.\n\n");
}
