// This may look like C code, but it is really -*- C++ -*-
//
// Implements the BE junction currents of a NPN BJT
//
//             
// 

#ifndef BJTBE_H
#define BJTBE_H 1

class BJTBE_eval {
public:
  // This are locally-defined variables
  enum {T_ISoBF, T_ISE, VT, MAX_TVAR_INDEX};
  enum {dummy, MAX_CVAR_INDEX};

  BJTBE_eval(int *iparm) {} // iparm not needed here

  // Main evaluation function. Only voltage dependence considered here.
  template <typename currDouble, typename voltDouble, typename paramDouble, 
	    typename varDouble, typename cvarDouble>
  inline void operator()(currDouble& ibe, voltDouble* v, 
			 paramDouble* dparm, 
			 varDouble* tvar, cvarDouble* cvar)
  {
    currDouble ibf_bf;
    getIexp(ibf_bf, v[0], tvar[T_ISoBF], dparm[NF], tvar[VT]);
    currDouble ile;
    getIexp(ile, v[0], tvar[T_ISE], dparm[NE], tvar[VT]);
    ibe = ibf_bf + ile;
    ibe *= dparm[AREA];
  }

  // Adjust variables to a new temperature.
  template <typename tvarDouble, typename varDouble, typename tempDouble>
  inline void setTemp(tvarDouble* tvar, 
		      varDouble* dparm, varDouble* cvar, tempDouble& T)
  {
    tempDouble Tabs = T + tzero;
    // Parameter calculation 
    getVt(tvar[VT], Tabs);
    getIs(tvar[T_ISoBF], dparm[IS], dparm[NF], tvar[VT], dparm, Tabs);
    getIs(tvar[T_ISE], dparm[ISE], dparm[NE], tvar[VT], dparm, Tabs);
    tvarDouble bf;
    getB(bf, dparm[BF], dparm, Tabs);
    tvar[T_ISoBF] /= bf;
  }
  
  // For efficiency, we may want to pre-calculate some expressions
  // that do not change with voltage or temperature.  Adjust variables
  // to a new parameter set.
  template <typename varDouble>
  inline void newParms(varDouble* cvar, varDouble* dparm)
  {
    // Do nothing.
  }

  // Calculate power. pout initially contains the current.
  template <typename powerDouble, typename voltDouble, 
	    typename paramDouble>
  inline void power(powerDouble& pout, voltDouble* v, 
		    paramDouble* dparm)
  {
    pout *= v[0];
  }

  //private:
  
  // Pointer to integer parameter vector from macroelement
  //  int *iparm; // not necessary for this model

};

// No thermal port
typedef FADBADGenericVCS<BJTBE_eval, GenericVCCS, 1, 
			 MAX_DPARM_INDEX, TEMP, false> BJTBE;
// With thermal port
typedef FADBADTGenericVCS<BJTBE_eval, GenericVCCS, 2, 
			  MAX_DPARM_INDEX, TEMP> TBJTBE;

// Power source
typedef FADBADPowerSource<BJTBE_eval, 2, 
 			  MAX_DPARM_INDEX, TEMP> BJTBE_PS;

#endif
