// This may look like C code, but it is really -*- C++ -*-
//
// Implements the CE current of a NPN BJT
//
//             
// 

#ifndef BJTCE_H
#define BJTCE_H 1

class BJTCE_eval {
public:
  // This are locally-defined variables
  enum {T_IS, VT, MAX_TVAR_INDEX};
  enum {dummy, MAX_CVAR_INDEX};

  BJTCE_eval(int *iparm) {} // iparm not needed here

  // Main evaluation function. Only voltage dependence considered here.
  template <typename currDouble, typename voltDouble, typename paramDouble, 
	    typename varDouble, typename cvarDouble>
  inline void operator()(currDouble& ice, voltDouble* v, 
			 paramDouble* dparm, 
			 varDouble* tvar, cvarDouble* cvar)
  {
    // v[0] : Vbe
    // v[1] : Vbc
    currDouble ibf;
    getIexp(ibf, v[0], tvar[T_IS], dparm[NF], tvar[VT]);
    currDouble ibr;
    getIexp(ibr, v[1], tvar[T_IS], dparm[NR], tvar[VT]);
    currDouble kqb;
    getKqb(kqb, v[0], v[1], ibf, ibr, dparm, tvar[T_IS], tvar[VT]);
    ice = dparm[AREA] * (ibf - ibr) / kqb;
  }

  // Adjust variables to a new temperature.
  template <typename tvarDouble, typename varDouble, typename tempDouble>
  inline void setTemp(tvarDouble* tvar, 
		      varDouble* dparm, varDouble* cvar, tempDouble& T)
  {
    tempDouble Tabs = T + tzero;
    // Parameter calculation 
    getVt(tvar[VT], Tabs);
    getIs(tvar[T_IS], dparm[IS], dparm[NF], tvar[VT], dparm, Tabs);
  }
  
  // For efficiency, we may want to pre-calculate some expressions
  // that do not change with voltage or temperature.  Adjust variables
  // to a new parameter set.
  template <typename varDouble>
  inline void newParms(varDouble* cvar, varDouble* dparm)
  {
    // Do nothing.
  }

  // Calculate power. pout initially contains the current.
  template <typename powerDouble, typename voltDouble, 
	    typename paramDouble>
  inline void power(powerDouble& pout, voltDouble* v, 
		    paramDouble* dparm)
  {
    pout *= (v[0]-v[1]);
  }

  //private:
  
  // Pointer to integer parameter vector from macroelement
  //  int *iparm; // not necessary for this model

};

// No thermal port
typedef FADBADGenericVCS<BJTCE_eval, GenericVCCS, 2, 
			 MAX_DPARM_INDEX, TEMP, false> BJTCE;
// With thermal port
typedef FADBADTGenericVCS<BJTCE_eval, GenericVCCS, 3, 
			  MAX_DPARM_INDEX, TEMP> TBJTCE;

// Power source
typedef FADBADPowerSource<BJTCE_eval, 3, 
 			  MAX_DPARM_INDEX, TEMP> BJTCE_PS;


#endif
