#include "Capacitor.h"
#include <carrot/FADBADLinearVCS.h>

static ItemInfo Capacitor_info = {
  "cap",
  "Linear Capacitor",
  "Carlos Christoffersen"
  "http://vision.lakeheadu.ca/"
};

// Double Parameters
enum {CAP, RINT, MAX_DPARM_INDEX};

//Insert ParmInfo values here
static ParamHandler<double>::ParmInfo Capacitor_dparams[] = { 
  {"c", "Capacitance (F)", true, 1e-9},
  {"rint", "Internal Resistance (Ohms)", false, 0.}
};

// enum {MAX_IPARM_INDEX};
// static ParamHandler<int>::ParmInfo Capacitor_iparams[] = {  };


//Library loading mechanism
extern "C" {
  const char * ModuleName(){
    return Capacitor_info.type.c_str();
  }

  ParmSet * newParmSet(const char * model){
    return new ParmSet(model,
		       &Capacitor_info,
		       Capacitor_dparams, MAX_DPARM_INDEX);
  }

  void delParmSet(ParmSet * p){
    delete p;
    p=0;
  }

  MacroElement * newObject(ParmSet * p, const char * name){
    return new Capacitor(p, name);
  }

  void delObject(MacroElement * e){
    delete e;
    e = 0;
  }
};

#include "CapType.h"
#include "RIType.h"

Capacitor::Capacitor(ParmSet *pset, const std::string& name)
  :MacroElement(pset, name, 2)
{
  //Constructor code goes here
}

Capacitor::~Capacitor()
{
  //Destructor code goes here
}

void Capacitor::init()
{
  //init code goes here
  double *dparm = getParmSet()->getDHandler()->getParmVec();
  // This flow should be similar for many MacroElements:
  //
  // 0. Check parameters and terminals
  // (We should really throw and exception in the following:)
  getParmSet()->checkParms();
  assert(checkTerms());
  // No internal nodes needed for now
  setNInternalNodes(0);
  // 1. Create individual CSs and add to vectors
  // 2. Assign internal terminals
  LinearVCQS* lvcqs = new CapType(dparm, NULL);
  static const int tvec[4] = {0,1,0,1};
  insert(lvcqs, tvec);
  if (dparm[RINT]) {
    LinearVCCS* lvccs = new RIType(dparm, NULL);
    insert(lvccs, tvec);
  }
}


