// This may look like C code, but it is really -*- C++ -*-
//
// Common variables used in the Curtice model for diode models
//


#ifndef CommonCurtice_H
#define CommonCurtice_H 1

// Intermediate results needed in several current sources

// Used for Igs, Igd
template<typename outDouble, typename voltDouble, 
	 typename paramDouble, typename varDouble>
inline void getIexp(outDouble& ig, voltDouble& v, 
		    varDouble& is, varDouble& ib0, varDouble& vt,
		    paramDouble* dparm)
{
  ig = is * (safe_exp(v / dparm[N] / vt) - 1.);
  ig -= ib0 * safe_exp(-(v+dparm[VBD]) /dparm[NR] /vt);
}

template<typename outDouble, typename tempDouble>
inline void getVt(outDouble& vt, tempDouble& T)
{
  vt = kBoltzman * T / eCharge;
}

// Used for IS, IB0
template<typename outDouble, typename paramDouble, 
	 typename varDouble, typename tempDouble>
void getIs(outDouble& is, varDouble& vt, 
	   paramDouble& IS, paramDouble& N, 
	   paramDouble* dparm, tempDouble& T)
{
  outDouble tn = T / (dparm[TNOM]+tzero);
  is = IS * exp((tn-1.) * dparm[EG] /N /vt) * pow(tn, dparm[XTI]/N);
}

template<typename outDouble, typename paramDouble, typename tempDouble>
inline void getEgap(outDouble& egap, paramDouble* dparm, tempDouble& T)
{
  egap = dparm[EG] - .000702 * T*T / (T+1108);
}

// Used for VBI
template<typename outDouble, typename paramDouble, 
	 typename varDouble, typename cvarDouble,
	 typename tempDouble>
void getVbi(outDouble& Vbi, varDouble& vt, cvarDouble& egapnom,
	    paramDouble* dparm, tempDouble& T)
{
  outDouble tn = T / (dparm[TNOM]+tzero);
  outDouble egap;
  getEgap(egap, dparm, T);
  Vbi = dparm[VBI]*tn - 3.*vt*log(tn) - tn * egapnom + egap;
}

// Used for CJS0, CJD0
template<typename outDouble, typename paramDouble, 
	 typename varDouble, typename tempDouble>
void getCbi(outDouble& Cbi, varDouble& vt,
	    varDouble& Vbi, paramDouble& CJ, 
	    paramDouble MJ, paramDouble* dparm, tempDouble& T)
{
  Cbi = 1. + MJ * (.0004*(T-dparm[TNOM]) + 1. - Vbi/dparm[VBI]);
  Cbi *= CJ; 
}

// Used for Qgd, Qgs
template<typename outDouble, typename voltDouble, typename paramDouble, 
	 typename varDouble, typename cvarDouble>
void getQj(outDouble& qj, voltDouble& v, 
	   varDouble& CJ, varDouble& VJ,
	   varDouble& k5, varDouble& k6, varDouble& k7, 
	   cvarDouble& k4, paramDouble MJ, paramDouble* dparm)
{
  if (CJ > 0.) {
    if (v < dparm[FCC] * VJ) {
      qj = k5 * (1. - pow(1. - v / VJ, k4));
    } else {
      qj = k6 * (((1. - dparm[FCC]*(1. + MJ)) * v + .5*MJ*v*v/VJ) - k7)
	+ k5 * (1. - pow(1.-dparm[FCC], k4));
    }
  } else 
    qj = 0.;
}

// Used for all depletion capacitances
template<typename outDouble, typename paramDouble, 
	 typename varDouble, typename cvarDouble>
void getk567(outDouble& k5, outDouble& k6, outDouble& k7,
	     varDouble& VJ, varDouble& CJ, 
	     cvarDouble& k4, paramDouble MJ, paramDouble* dparm)
{
  k5 = VJ * CJ / k4;
  k6 = CJ * pow(1. - dparm[FCC], - MJ - 1.);
  k7 = ((1. - dparm[FCC] * (1.+MJ)) * VJ * dparm[FCC] 
	+ .5*MJ * VJ*dparm[FCC]*dparm[FCC]);
}


#endif
