// This may look like C code, but it is really -*- C++ -*-
//
// Implements the charge stored in the GD junction
//
//             
//          o  1              
//          |                            
//        --+--
//               QBC
//        --+--
//          |                          
//          o  0    	      
//                
// 

#ifndef QgdCurtice_H
#define QgdCurtice_H 1

class QgdCurtice_eval {
public:
  // This are locally-defined variables
  enum {VT, T_VBI, T_CGD, k5, k6, k7, MAX_TVAR_INDEX};
  enum {k4, EGAPn, MAX_CVAR_INDEX};

  QgdCurtice_eval(int *iparm) {} // iparm not needed here

  // Main evaluation function. Only voltage dependence considered here.
  template <typename currDouble, typename voltDouble, typename paramDouble, 
	    typename varDouble, typename cvarDouble>
  inline void operator()(currDouble& qgd, voltDouble* v, 
			 paramDouble* dparm, 
			 varDouble* tvar, cvarDouble* cvar)
  {
    // Depletion charge
    getQj(qgd, v[0], tvar[T_CGD], tvar[T_VBI],
	  tvar[k5], tvar[k6], tvar[k7], cvar[k4], dparm[MGD], dparm);
    // Area effect
    qgd *= dparm[AREA];
  }

  // Adjust variables to a new temperature.
  template <typename tvarDouble, typename varDouble, typename tempDouble>
  inline void setTemp(tvarDouble* tvar, 
		      varDouble* dparm, varDouble* cvar, tempDouble& T)
  {
    tempDouble Tabs = T + tzero;
    // Parameter calculation 
    getVt(tvar[VT], Tabs);
    getVbi(tvar[T_VBI], tvar[VT], cvar[EGAPn], dparm, Tabs);
    getCbi(tvar[T_CGD], tvar[VT], 
	   tvar[T_VBI], dparm[CGD0], dparm[MGD], dparm, Tabs);
    getk567(tvar[k5], tvar[k6], tvar[k7], 
	    tvar[T_VBI], tvar[T_CGD],
	    cvar[k4], dparm[MGD], dparm);
  }
  
  // For efficiency, we may want to pre-calculate some expressions
  // that do not change with voltage nor temperature.  Adjust variables
  // to a new parameer set.
  template <typename varDouble>
  inline void newParms(varDouble* cvar, varDouble* dparm)
  {
    cvar[k4] = 1. - dparm[MGD];
    varDouble tnom = dparm[TNOM]+tzero;
    getEgap(cvar[EGAPn], dparm, tnom);
  }

  //private:
  
  // Pointer to integer parameter vector from macroelement
  //  int *iparm; // not necessary for this model

};


typedef FADBADGenericVCS<QgdCurtice_eval, GenericVCQS, 1,
			 MAX_DPARM_INDEX, TEMP, false> QgdCurtice;
// With thermal port
typedef FADBADGenericVCS<QgdCurtice_eval, GenericVCQS, 2, 
			 MAX_DPARM_INDEX, TEMP, true> TQgdCurtice;

#endif
