// This may look like C code, but it is really -*- C++ -*-
//
// PMOS Bulk-Drain Capacitance implementation 
//
//          
// 

#ifndef Qdbp_H
#define Qdbp_H 1

class Qdbp_eval {
public:
  // This are locally-defined variables
  enum {DUMMY,MAX_TVAR_INDEX};
  enum {K1, K2, K3, K4, K5, K6, K7, K8, K9, MAX_CVAR_INDEX};

  Qdbp_eval(int *iparm) {} // iparm not needed here

  // v[0] = Vdb

  // Main evaluation function. Only voltage dependence considered here.
  template <typename currDouble, typename voltDouble, typename paramDouble, 
	    typename varDouble, typename cvarDouble>
  inline void operator()(currDouble& qdbp, voltDouble* v, 
			 paramDouble* dparm, 
			 varDouble* tvar, cvarDouble* cvar)
  {
  
    
	
	  
    		if (v[0] < cvar[K9]) {
    
    			qdbp = dparm[AD]*dparm[CJ]*cvar[K2]*(1.-pow(1.-v[0]/dparm[PB], cvar[K1]));
			qdbp += dparm[PD]*dparm[CJSW]*(cvar[K6]*(1.-pow(1.-v[0]/dparm[PBSW], cvar[K5])));
	
    		}else {
	  
	  		currDouble value1 = cvar[K4]*(v[0]-cvar[K9]);
	  			   value1 += dparm[MJ]/(2.*dparm[PB])*(v[0]*v[0]-cvar[K9]*cvar[K9]);
	  	     		   value1 *= dparm[AD]*dparm[CJ]*cvar[K3];
	  		currDouble value2 =  cvar[K8]*(v[0]-cvar[K9]);
	  			   value2 += dparm[MJSW]/(2.*dparm[PBSW])*(v[0]*v[0]-cvar[K9]*cvar[K9]);
	   	     		   value2 *= dparm[PD]*dparm[CJSW]*cvar[K7];
	   	     
	   	     	qdbp = value1+value2;
		   	         
		 	 
		}
}

  // Adjust variables to a new temperature.
  template <typename tvarDouble, typename varDouble, typename tempDouble>
  inline void setTemp(tvarDouble* tvar, 
		      varDouble* dparm, varDouble* cvar, tempDouble& T)
  {
  
  }
  
  // For efficiency, we may want to pre-calculate some expressions
  // that do not change with voltage nor temperature.  Adjust variables
  // to a new parameer set.
  template <typename varDouble>
  inline void newParms(varDouble* cvar, varDouble* dparm)
  {
  
  cvar[K1] = 1.-dparm[MJ];
  cvar[K2] = dparm[PB]/cvar[K1];
  cvar[K3] = pow(1.-dparm[FC], -(1.+dparm[MJ]));
  cvar[K4] = 1.-dparm[FC]*(1.+dparm[MJ]);
  cvar[K5] = 1.-dparm[MJSW];
  cvar[K6] = dparm[PBSW]/cvar[K5];
  cvar[K7] = pow(1.-dparm[FC], -(1.+dparm[MJSW]));
  cvar[K8] = 1.-dparm[FC]*(1.+dparm[MJSW]);
  cvar[K9] = dparm[FC]*dparm[PB];
  //cvar[AD_1] = 0.48e-6*dparm[W];
  //cvar[PD_1] = 0.96e-6+dparm[W];   
  
  }

  //private:
  
  // Pointer to integer parameter vector from macroelement
  //  int *iparm; // not necessary for this model

};


typedef FADBADGenericVCS<Qdbp_eval, GenericVCQS, 1, 
			 MAX_DPARM_INDEX, TEMP, false> Qdbp;
// With thermal port
//typedef FADBADTGenericVCS<DiodeCharge_eval, GenericVCQS, 2, 
//			  MAX_DPARM_INDEX, TEMP> TDiodeCharge;

#endif
