// This may look like C code, but it is really -*- C++ -*-
//
// InternalMSinCS is a DC + multi-time sinusoidal current source
//
//          1  
//          o        
//          |       
//          |       
//         (^) J
//          |       
//          |     
//          o       
//          2  
//
//
// by Carlos E. Christoffersen

#ifndef InternalMSinCS_h
#define InternalMSinCS_h 1

class InternalMSinCS_eval {
public:
  // This are locally-defined variables 
  enum {OMEGA, CAMP, MAX_CVAR_INDEX};

  InternalMSinCS_eval(int *iparm) {} // iparm not needed here

  // Main evaluation function. Only voltage dependence considered here.
  template <typename currDouble, typename paramDouble, typename cvarDouble>
  inline void operator()(currDouble& iout, 
			 const double& ctime1, const double& ctime2, 
			 paramDouble* dparm, cvarDouble* cvar)
  {
    paramDouble factor = 1.;
    // Rise time treatment
    if (ctime1 < dparm[TR])
      factor = ctime1 / dparm[TR];
    // Treat AM parameter (equivalent to BPSK at this moment)
    if (dparm[AMT] != 0.) {
      paramDouble amf = dparm[AMF] * sin(twopi * ctime1 / dparm[AMT]);
      if (amf > 1.)
	amf = 1.;
      else if (amf < -1.)
	amf = -1;
      amf += dparm[CARRIER];
      factor *= amf;
    }
    iout = factor * cvar[CAMP] * sin(cvar[OMEGA]*ctime2 + dparm[PHASE]);
  }

  // For efficiency, we may want to pre-calculate some expressions
  // that do not change with voltage or temperature.  Adjust variables
  // to a new parameer set.
  template <typename varDouble>
  inline void newParms(varDouble* cvar, varDouble* dparm)
  {
    cvar[OMEGA] = twopi * dparm[FREQ];
    if (dparm[RI] != 0.)
      cvar[CAMP] = dparm[AMP] / dparm[RI];
    else {
      const double &scale = options.getDHandler()->getParmVec()[OPT_SCALE];
      cvar[CAMP] = dparm[AMP] * scale;
    }
  }

  //private:
  
  // Pointer to integer parameter vector from macroelement
  //  int *iparm; // not necessary for this model

};

typedef FADBADMIndependentCS<InternalMSinCS_eval, MAX_DPARM_INDEX> 
InternalMSinCS;


#endif


