#include "NLCap.h"
#include "../global.h"
#include <carrot/Options.h>
#include <carrot/FADBADGenericVCS.h> 

static ItemInfo NLCap_info = {
  "nlcap",
  "Generic nonlinear capacitor",
  "Carlos Christoffersen"
  DEFAULT_ADDRESS
};

// Double Parameters
// Make sure enum and ParInfo definitions are in the same order!
enum {C0, V0, TEMP, MAX_DPARM_INDEX};

//Insert ParmInfo values here
static ParamHandler<double>::ParmInfo NLCap_dparams[] = { 
  {"c0", "Capacitance multiplier (F)", false, 224.9e-12},
  {"v0", "Threshold potential", false, 3.73},
  {"temp", "Temperature (C)", false, 27.}
};

// Uncomment for integer parameters
//enum {MAX_IPARM_INDEX};
//static ParamHandler<int>::ParmInfo NLCap_iparams[] = {  };

#include "QNLCap.h"

//Library loading mechanism
extern "C" {
  const char * ModuleName(){
    return NLCap_info.type.c_str();
  }
  ParmSet * newParmSet(const char * model){
    return new ParmSet(model,
		         &NLCap_info,
		         NLCap_dparams, MAX_DPARM_INDEX); 
//		         NLCap_iparams, MAX_IPARM_INDEX);
  }
  void delParmSet(ParmSet * p){
    delete p;
    p=0;
  }
  MacroElement * newObject(ParmSet * p, const char * name){
    return new NLCap(p, name);
  }
  void delObject(MacroElement * e){
    delete e;
    e = 0;
  }
};

NLCap::NLCap(ParmSet *pset, const std::string& name)
  :MacroElement(pset, name, 2)
{
  //Constructor code goes here (above, 2 is the number of terminals)
}

NLCap::~NLCap()
{
  //Destructor code goes here
}

void NLCap::init()
{
  // access to double parameters
  double *dparm = getParmSet()->getDHandler()->getParmVec();
  // Check required parameters
  getParmSet()->checkParms();
  // Check external terminal connections
  assert(checkTerms());
  // Set number of internal nodes
  setNInternalNodes(0);

  GenericVCQS* qcap = new QNLCap(dparm, NULL);
  static const int qterms[4] = {0,1,0,1};	
  insert(qcap, qterms);

}

