// This may look like C code, but it is really -*- C++ -*-
//
// Implementation of nonlinear capacitor. Change equation here.
//
//             
//          o  1              
//          |                            
//        --+--
//               QNLCap
//        --+--
//          |                          
//          o  0    	      
//                
// 

#ifndef QNLCap_H
#define QNLCap_H 1

class QNLCap_eval {
public:
  // This are locally-defined variables
  enum {DUMMY1, MAX_TVAR_INDEX};
  enum {DUMMY2, MAX_CVAR_INDEX};

  QNLCap_eval(int *iparm) {} // iparm not needed here

  // Main evaluation function. Only voltage dependence considered here.
  template <typename currDouble, typename voltDouble, typename paramDouble, 
	    typename varDouble, typename cvarDouble>
  inline void operator()(currDouble& q, voltDouble* v, 
			 paramDouble* dparm, 
			 varDouble* tvar, cvarDouble* cvar)
  {
    const paramDouble vmin = -.99 * dparm[V0];
    currDouble v1 = 1. + v[0] / dparm[V0];
    if (v1 < .01)
      v1 = .01;
    q = dparm[C0] * dparm[V0] * log(v1);
    if (v[0] < vmin)
      q *= (vmin - v[0]);

//    const double v0 = .6;
//    const double v1 = -.9999 * v0;
//    if (v[0] > v1) {
//      q = dparm[C0] * v[0];
//      q /= sqrt(1 + v[0] / v0);
//    } else {
//      q = dparm[C0] * v[0];
//      q /= sqrt(1 + v1 / v0);
//    }
  }

  // Adjust variables to a new temperature.
  template <typename tvarDouble, typename varDouble, typename tempDouble>
  inline void setTemp(tvarDouble* tvar, 
		      varDouble* dparm, varDouble* cvar, tempDouble& T)
  {
    // Do nothing
  }
  
  // For efficiency, we may want to pre-calculate some expressions
  // that do not change with voltage nor temperature.  Adjust variables
  // to a new parameer set.
  template <typename varDouble>
  inline void newParms(varDouble* cvar, varDouble* dparm)
  {
    // Do nothing
  }

  //private:
  
  // Pointer to integer parameter vector from macroelement
  //  int *iparm; // not necessary for this model

};


typedef FADBADGenericVCS<QNLCap_eval, GenericVCQS, 1, 
			 MAX_DPARM_INDEX, TEMP, false> QNLCap;

#endif
