// This may look like C code, but it is really -*- C++ -*-
//
// InternalDCCS is a DC current source
//
//          1  
//          o        
//          |       
//          |       
//         (^) J
//          |       
//          |     
//          o       
//          2  
//
//
// by Carlos E. Christoffersen

#ifndef InternalDCCS_h
#define InternalDCCS_h 1

class InternalDCCS_eval {
public:
  // This are locally-defined variables 
  enum {CVAL, MAX_CVAR_INDEX};

  InternalDCCS_eval(int *iparm) {} // iparm not needed here

  // Main evaluation function. Only voltage dependence considered here.
  template <typename currDouble, typename paramDouble, typename cvarDouble>
  inline void operator()(currDouble& iout, const double& ctime, 
			 paramDouble* dparm, cvarDouble* cvar)
  {
    iout = cvar[CVAL];
  }

  // For efficiency, we may want to pre-calculate some expressions
  // that do not change with voltage or temperature.  Adjust variables
  // to a new parameer set.
  template <typename varDouble>
  inline void newParms(varDouble* cvar, varDouble* dparm)
  {
    if (dparm[RI] != 0.)
      cvar[CVAL] = dparm[VAL] / dparm[RI];
    else {
      const double &scale = options.getDHandler()->getParmVec()[OPT_SCALE];
      cvar[CVAL] = dparm[VAL] * scale;
    }
  }

  //private:
  
  // Pointer to integer parameter vector from macroelement
  //  int *iparm; // not necessary for this model

};

typedef FADBADIndependentCS<InternalDCCS_eval, MAX_DPARM_INDEX> InternalDCCS;


#endif


