// This may look like C code, but it is really -*- C++ -*-
//
// InternalDCCS is a DC current source
//
//          1  
//          o        
//          |       
//          |       
//         (^) J
//          |       
//          |     
//          o       
//          2  
//
//
// by Carlos E. Christoffersen

#ifndef InternalPulseCS_h
#define InternalPulseCS_h 1

class InternalPulseCS_eval {
public:
  // This are locally-defined variables 
  enum {CVAL1, CVAL2, MAX_CVAR_INDEX};

  InternalPulseCS_eval(int *iparm) {} // iparm not needed here

  // Main evaluation function. Only voltage dependence considered here.
  template <typename currDouble, typename paramDouble, typename cvarDouble>
  inline void operator()(currDouble& iout, const double& ctime, 
			 paramDouble* dparm, cvarDouble* cvar)
  {
    if (ctime < dparm[TR]) {
      iout = cvar[CVAL1] + (cvar[CVAL2]-cvar[CVAL1]) * ctime / dparm[TR];
    } else if (ctime < dparm[PULSEDT]) {
      iout = cvar[CVAL2];
    } else if (ctime < dparm[PULSEDT] + dparm[TF]) {
      iout = cvar[CVAL2] + (cvar[CVAL1]-cvar[CVAL2]) * 
	(ctime - dparm[PULSEDT])  / dparm[TF];
    } else
      iout = cvar[CVAL1];
  }

  // For efficiency, we may want to pre-calculate some expressions
  // that do not change with voltage or temperature.  Adjust variables
  // to a new parameer set.
  template <typename varDouble>
  inline void newParms(varDouble* cvar, varDouble* dparm)
  {
    if (dparm[RI] != 0.) {
      cvar[CVAL1] = dparm[V1] / dparm[RI];
      cvar[CVAL2] = dparm[V2] / dparm[RI];
    } else {
      const double &scale = options.getDHandler()->getParmVec()[OPT_SCALE];
      cvar[CVAL1] = dparm[V1] * scale;
      cvar[CVAL2] = dparm[V2] * scale;
    }
  }

  //private:
  
  // Pointer to integer parameter vector from macroelement
  //  int *iparm; // not necessary for this model

};

typedef FADBADIndependentCS<InternalPulseCS_eval, MAX_DPARM_INDEX> 
InternalPulseCS;


#endif


