// This may look like C code, but it is really -*- C++ -*-
//
// InternalSinCS is a DC + sinusoidal current source
//
//          1  
//          o        
//          |       
//          |       
//         (^) J
//          |       
//          |     
//          o       
//          2  
//
//
// by Carlos E. Christoffersen

#ifndef InternalSinCS_h
#define InternalSinCS_h 1

class InternalSinCS_eval {
public:
  // This are locally-defined variables 
  enum {OMEGA, CAMP, MAX_CVAR_INDEX};

  InternalSinCS_eval(int *iparm) {} // iparm not needed here

  // Main evaluation function. Only voltage dependence considered here.
  template <typename currDouble, typename paramDouble, typename cvarDouble>
  inline void operator()(currDouble& iout, const double& ctime, 
			 paramDouble* dparm, cvarDouble* cvar)
  {
    iout = cvar[CAMP] * sin(cvar[OMEGA]*ctime + dparm[PHASE]);
  }

  // For efficiency, we may want to pre-calculate some expressions
  // that do not change with voltage or temperature.  Adjust variables
  // to a new parameer set.
  template <typename varDouble>
  inline void newParms(varDouble* cvar, varDouble* dparm)
  {
    cvar[OMEGA] = twopi * dparm[FREQ];
    if (dparm[RI] != 0.)
      cvar[CAMP] = dparm[AMP] / dparm[RI];
    else {
      const double &scale = options.getDHandler()->getParmVec()[OPT_SCALE];
      cvar[CAMP] = dparm[AMP] * scale;
    }
  }

  //private:
  
  // Pointer to integer parameter vector from macroelement
  //  int *iparm; // not necessary for this model

};

typedef FADBADIndependentCS<InternalSinCS_eval, MAX_DPARM_INDEX> InternalSinCS;


#endif


