// This may look like C code, but it is really -*- C++ -*-
//
// Template class for vectors in the Circuit Class
//
// by Carlos E. Christoffersen

#ifndef NodeVector_h
#define NodeVector_h 1

#include <vector>
#include <cassert>

template <typename T, typename TV>
class NodeVector 
{
public:

  NodeVector() : vec(), flag(false) {}
  
  ~NodeVector() {}

  inline void add(T elem)
  {
    // Make sure it is not already inserted
    assert(elem->getIndex() == -1);
    elem->setIndex(vec.size());
    vec.push_back(elem);
  }

  inline void remove(T elem)
  {
    // Make sure the element is in place
    assert(elem == vec[elem->getIndex()]);
    // Set the pointer to NULL and fix vector later
    vec[elem->getIndex()] = NULL;
    // Disable index in element
    elem->setIndex(-1);
    // Set flag for update
    flag = true;
  }

  inline T& getElem(const unsigned& i)
  {
    return vec[i];
  }

  inline const unsigned size()
  {
    if (flag)
      updateState();
    return vec.size();
  }

  inline TV& getVec()
  {
    if (flag)
      updateState();
    return vec;
  }

  void eraseElements()
  {
    int imax(vec.size());
    for (int i(0); i < imax; i++) 
      // Check in case some pointers are NULL
      if (vec[i])
	delete vec[i];
    vec.clear();
  }
  
private:

  void updateState()
  {
    unsigned iend = vec.size();
    for (unsigned i(0); i < iend; i++) {
      if (vec[i] == NULL) {
	// Search for last non-NULL element
	while (vec.back() == NULL) {
	  vec.pop_back();
	  iend--;
	}
	// Exchange only if there is anything left to exchange
	if (i < iend-1) {
	  vec[i] = vec.back();
	  vec.pop_back();
	  vec[i]->setIndex(i);
	}
      }
    }
    flag = false;
  }

  TV vec;
  bool flag;

};


#endif


