// This may look like C code, but it is really -*- C++ -*-
//
// Manage simulation results storage and output.
//
// by Carlos E. Christoffersen

#ifndef OutputManager_h
#define OutputManager_h 1

#include <carrot/Circuit.h>
#include <carrot/OP.h>
#include <carrot/Options.h>
#include <carrot/SampleHolder.h>

#include <string>
#include <cassert>
//#include <iostream>

class OutputManager
{
public:

  OutputManager();
  
  ~OutputManager(); 

  // Select nodes that must be saved (default: all)
  inline void addNode(Terminal* term)
  {
    assert(!ready);
    tv.push_back(term);
  }
  
  // set unit of main variable (transient:time, DC:sweep variable)
  inline void setVarUnit(const std::string& unit)
  {
    varunit = unit;
  }

  // set the analysis type (for identification)
  inline void setAnalysis(const std::string& name)
  {
    analysistype = name;
  }

  // Inquire about voltages to be saved (row and column numbers must
  // be already set in terminals before calling this).
  void getRowNumbers(Circuit* cir, std::vector<int>& iv);

  // Add a new sample. OutputManager will take care of deleting sh
  // (with delete sh) so *sh can not be allocated in the stack.
  inline void addSample(SampleHolder* sh)
  {
    assert(ready);    
    assert(sh);
    sv.push_back(sh);
  }

  // Perform output (to file) for selected variables
  void write();

private:

  // Flag to remember if the terminal vector has already been ordered
  // and things are ready to start inserting samples.
  bool ready;

  // Vector with terminals selected for output
  TerminalVector tv;
  // Unit of independent variable 
  std::string varunit;
  // Analysis name (for identification)
  std::string analysistype;
  // Vector with the actual data
  std::vector<SampleHolder*> sv;
};


#endif


