// This may look like C code, but it is really -*- C++ -*-
//
// Class to hold terminals (the `real' nodes of the circuit). The
// order of the elements in elemVec is not important in principle.
//
// by Carlos E. Christoffersen

#ifndef Terminal_h
#define Terminal_h 1

#include <carrot/CircuitItem.h>
#include <carrot/NamedItem.h>
#include <vector>
#include <string>

// Defined in ElementNode.h
class ElementNode;

class Terminal : public CircuitItem, public NamedItem
{
public:

  Terminal(const std::string& name, bool intern = false);
  
  ~Terminal();

  // Link to an element (add element to list)
  // Solve the problem of repetitions. Should we check?
  inline void link(ElementNode* en)
  {
    elemVec.push_back(en);
  }

  void unlink(ElementNode* en);

  // Return number of connections
  inline unsigned getNConnections() const
  {
    return elemVec.size();
  }

  inline bool isInternal() const
  {
    return internal;
  }

  // Get element by index
  inline ElementNode* getGNode(const unsigned& i)
  {
    assert(i < elemVec.size());
    return elemVec[i];
  }
  
  // Row/Column number
  inline void setRC(const int& rc)
  {
    myRC = rc;
  }

  inline const int& getRC() const
  {
    return myRC;
  }
  
private:

  // By default: -1 (points to ground)
  int myRC;
  // List of nodes connected
  std::vector<ElementNode*> elemVec;
  // Flag to signal if the terminal is internal in a MacroElement
  bool internal;
};

typedef std::vector<Terminal*> TerminalVector;

#endif


